Message
send(Message query) throws IOException {
	byte [] out, in;
	Message response;
	DatagramSocket s;
	DatagramPacket dp;
	int udpLength = 512;

	try {
		s = new DatagramSocket();
	}
	catch (SocketException e) {
		System.out.println(e);
		return null;
	}

	if (EDNSlevel >= 0) {
		udpLength = 1280;
		query.addRecord(Section.ADDITIONAL, EDNS.newOPT(udpLength));
	}

	if (tsig != null)
		tsig.apply(query, null);


	out = query.toWire();

	if (useTCP)
		return sendTCP(query, out);

	s.send(new DatagramPacket(out, out.length, addr, port));

	dp = new DatagramPacket(new byte[udpLength], udpLength);
	s.setSoTimeout(timeoutValue);
	try {
		s.receive(dp);
	}
	catch (InterruptedIOException e) {
		s.close();
		System.out.println(";; No response");
		return null;
	}
	in = new byte [dp.getLength()];
	System.arraycopy(dp.getData(), 0, in, 0, in.length);
	response = new Message(in);
	if (tsig != null) {
		boolean ok = tsig.verify(response, in, query.getTSIG());
		System.out.println(";; TSIG verify: " + ok);
	}

	s.close();
	if (response.getHeader().getFlag(Flags.TC) && !ignoreTruncation)
		return sendTCP(query, out);
	else
		return response;
}
